
#include "proto.h"
#include "preprocess.h"

#define ASM_INFO              0
#define ASM_WARNING           1
#define ASM_ERROR             2

#define SYMBOL_CONSTANT       1
#define SYMBOL_LABEL          2
#define SYMBOL_BRANCHLABEL    4
#define SYMBOL_IMPORTED       (1<<8)
#define SYMBOL_EXPORTED       (1<<9)
#define SYMBOL_UNUSED         (1<<16)


#define MAXSYMBOLS            1024
#define MAXERRORLENGTH        128
#define MAXERRORS             32

typedef struct SYMBOL SYMBOL;
typedef struct ASM ASM;
typedef struct ASMERROR ASMERROR;


struct ASMERROR {
  U32 lineno;
  U16 fileno;
  char errmsg[MAXERRORLENGTH];
};


struct SYMBOL {
  char *name;
  int value;
  U32 flags;
};

#ifdef __cplusplus
extern "C" {
#endif

// call assembler() to assemble the lines
ASM *assembler(LINE *lines, U32 nlines, void (* callback)(char *, LINE *, U32));

// call assembler_errors() to read the list of errors generated by the assembler
U32 assembler_errors(ASM *assm, ASMERROR **errors);

// call assembler_output() to read info about the output from assembler()
void assembler_output(ASM *assm, U32 *size, void **output);

// call assembler_symbols() to read info about the symbols defined by assembler()
void assembler_symbols(ASM *assm, SYMBOL **symbols, U32 *nsymbols);

// call assembler_kill() to release the workspace used by assembler()
void assembler_kill(ASM *assm);
#ifdef __cplusplus
}
#endif
