// main.h
//


//////////////////////////////////////////////////////////////
//
// This file contains class definitions for the asm program.
//
//////////////////////////////////////////////////////////////
#ifndef __ASMMODEL_H
#define __ASMMODEL_H

#if !defined(__S32FILE_H__)
#include <s32file.h>
#endif

#include <apparc.h>

#include <coecntrl.h>
#include <coeccntx.h>
#include <coemain.h>
#include <coeutils.h>

#include <eikappui.h>
#include <eikapp.h>
#include <eikcfdlg.h>
#include <eikcmds.hrh>
#include <eikdef.h>
#include <eikdoc.h>
#include <eikenv.h>
#include <eikfnlab.h>
#include <eikkeys.h>
#include <eiklabel.h>
#include <eikmenup.h>
#include <eikmenu.hrh>
#include <eikproc.h>
#include <eikscrlb.h>
#include <eiktbar.h>
#include <eiktbar.hrh>
#include <eikon.rsg>

#include <e32std.h>
#include <s32stor.h>
#include <f32file.h>
#include <txtetext.h>
#include <txtfrmat.h>
#include <txtglobl.h>
#include <txtfmlyr.h>
#include <frmtview.h>

#include <asm.rsg>
#include "asm.hrh"

#include "asm.h"

// UID of app
const TUid KUidAsm = { 0x1000a5cf };

class CAsmDocument;
class CAsmAppView;
class CAsmAppUi;

//
// class CAsmAppView
//

class CAsmAppView : public CCoeControl, public MEikScrollBarObserver {
  public:
    void ConstructL(const TRect& aRect, CAsmDocument *aDoc, CAsmAppUi *aAppUi);
    ~CAsmAppView();

    TBool DoCreateNewFile(const TFileName& aFileName);
    TBool DoOpenFile(const TFileName& aFileName);
    TBool DoSaveFileAssembled(const TFileName& aFileName);
    TBool DoSaveFile();
    TBool DoSaveFileAs(const TFileName& aFileName);
    TFileName FileName() { return iFileName; };
    TFileName IncludeFile() { return iIncludeFile; };
    void CreateComponentControlsL();
    void CreateScrollBarL();
    void RemoveScrollBarL();
    void DynInitMenuPaneL(TInt aMenuId, CEikMenuPane* aMenuPane);

    void UpdateScrollBar(TInt aYChange, TBool aForce);
    void UpdateParagraph();
    void SetZoom(TInt aChange, TBool aUpdateScrollBars);
    void ToggleToolBar();

    void Cut();
    void Copy();
    void Paste();

    void Assemble();
    void AddToListOfRecentFiles(const TFileName& aFileName);
    void RemoveFromListOfRecentFiles(const TFileName& aFileName);
    void OpenRecentFile(TInt iWhich);
    void InfoWindow(const TDesC& aTitle, const TDesC& aText);
    void GotoError(TInt aDirection);
  private: // from CCoeControl
    void HandlePointerEventL(const TPointerEvent& aPointerEvent);
    TKeyResponse OfferKeyEventL(const TKeyEvent& aKeyEvent,TEventCode aType);
    void Draw(const TRect& /*aRect*/) const;

    void ClearSelection();
    TPoint SetSelection(TInt aPos);
    TPoint DeleteSelection();

    void UpdateErrorPointers(TInt aPos, TInt aInserted);

    void GotoLineNo(TInt aLineNo);
//    void SetInfoFileName(const TDesC& aMessage);
//    void SetInfoLineNo(const TInt aLineNo);
//    void SetInfoLineCount(const TInt aLineCount);
//    void SetInfoPrevLabel(const TDesC& aMessage);

  private: // from MEikScrollBarObserver
    void HandleScrollEventL(CEikScrollBar* aScrollBar, TEikScrollEvent aEvent);

  public:
    TBool iActivated;
    TFileName iSavedAsmFileName;
    TInt iRecentFilesCount;
    TInt iScreenSizeX, iScreenSizeY;

  private:
    CAsmDocument *iDocument;
    CAsmAppUi *iAppUi;
    // misc
    TBool iHasChanged;
    TFileName iFileName;
    TFileName iRecentFiles[5];
    // global text
    CGlobalText *iGlobalText;
    CParaFormatLayer *iParaFormatLayer;
    CCharFormatLayer *iCharFormatLayer;
    // text layout and view
    CTextLayout *iLayout;
    CTextView *iTextView;
    TRect iViewRect, iViewRectSource;
    TInt iScrolled;
    CFbsBitmap *iBitmap;
    // text editing
    TInt iPos;
    TInt iPreferredPosOnLine;
    TInt iParagraphNo;
    TInt iParagraphStartPos;
    TInt iParagraphCount;
    TFileName iIncludeFile;
    // window stuff
    RWindowGroup iGroup;
    CEikScrollBar *iVBar;
    TInt iThumbPosition, iHeight,iScrollSpan;
    TInt iZoom;
    // output
    TAny *iAssembledOutput;
    TInt iAssembledSize;
    TFileName iLibrary;
    // asm errors for this file
    TInt iAsmErrorCount;
    TInt iAsmErrorPos[20];
    TBuf8<128> iAsmErrors[20];
    // selection
    TInt iSelectionStart, iSelectionEnd;
    HBufC *iClipboard;
    TInt iClipboardSize;
    // information
//    CEikLabel *iInfoFileName, *iInfoPrevLabel;
//    CEikLabel *iInfoLineNo, *iInfoLineCount;
};

//
// CAsmAppUi
//

class CAsmAppUi : public CEikAppUi {
  public:
    void ConstructL();
    ~CAsmAppUi();

  private: // from CEikAppUi
    void HandleCommandL(TInt aCommand);
    void SetInitialPathL(TFileName& aFileName);
    void CmdFileOpenL();
    void OpenFileL(TFileName& aFileName);
    void CmdFileSaveL();
    void SaveFileL(TFileName& aFileName);
    void CmdFileSaveAssembledL();
    void SaveFileAssembledL(TFileName& aFileName);
    void CmdFileNewL();
    void CreateFileL(TFileName& aFileName);
    void ModelHasChanged();
    void DynInitMenuPaneL(TInt aMenuId, CEikMenuPane* aMenuPane);

  public:
    TBool iToolBarVisible;

  private:
    CAsmAppView* iAppView;
    void OpenFileFromSameDir(TFileName& aBase, TFileName& aFileName);

};

//
// CAsmDocument
//

class CAsmDocument : public CEikDocument {
  public:
    // construct/destruct
    CAsmDocument(CEikApplication& aApp);
    ~CAsmDocument();
    void ConstructL();
    static CAsmDocument* NewL(CEikApplication& aApp);
    void ResetModelL();
 private: // from CApaDocument
    void StoreL(CStreamStore& aStore,CStreamDictionary& aStreamDic);
    void RestoreL(const CStreamStore& aStore,const CStreamDictionary& aStreamDic);
    void NewDocumentL();
  private: // from CEikDocument
    CEikAppUi* CreateAppUiL();
};

//
// CAsmApplication
//

class CAsmApplication : public CEikApplication {
  private: // from CApaApplication
    CApaDocument* CreateDocumentL();
    TUid AppDllUid() const;
};

#endif
